SystemOrganization addCategory: #LostInMorvanChat!
SystemOrganization addCategory: 'LostInMorvanChat-Client'!
SystemOrganization addCategory: 'LostInMorvanChat-Message'!
SystemOrganization addCategory: 'LostInMorvanChat-Server'!
SystemOrganization addCategory: 'LostInMorvanChat-User'!

Object subclass: #ChatClient
	instanceVariableNames: 'url login exit messages console lastMessageIndex networkError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClient commentStamp: 'FrancoisJarriges 6/2/2016 15:05' prior: 0!
I am a chat client. I'm sending my own messages  to an HTTP server and getting all the messages sent by all the chat clients including me to display on a console. 

ChatClient connect: 'localhost' port: 8080 login: 'toto'.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	console:		<Object>
	exit:		<Object>
	lastMessageIndex:		<Object>
	login:		<Object>
	messages:		<Object>
	networkError:		<Object>
	url:		<Object>


    Implementation Points!

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:19'!
connect: aHost port: aPort login: aLogin

	^ self new
		host: aHost port: aPort login: aLogin;
		start
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 22:38'!
basicSecuredGet: commandString
	
	| responseString |
	
	[ responseString := ZnClient new 
		url: commandString; 
		headerAt: 'ChatUserId' put: login asCamelCase uncapitalized;
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			responseString := nil ].
	responseString ifNotNil: [ networkError := nil ].
	^ responseString 
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 22:38'!
basicSecuredPost: commandString message: newChatMessage

	| responseString |

	[ responseString := ZnClient new
		url: commandString;
		headerAt: 'ChatUserId' put: login asCamelCase uncapitalized;
		entity: (ZnEntity 
              with: (STON toString: newChatMessage)
              type: ZnMimeType applicationSton);
		post ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			responseString := nil ].
	responseString ifNotNil: [ networkError := nil ].	
	^ responseString! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath

	^ '{1}{2}' format: { url . aPath }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath argument: anArgument

	^ '{1}{2}{3}' format: { url . aPath . anArgument asString }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 13:16'!
commandAddMessage

	^ self command: '/messages/add'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:34'!
commandLastMessageIndex

	^ self command: '/messages/count'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 14:31'!
commandMessagesFromLastIndex

	^ self command: '/messages/' argument: lastMessageIndex ! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/4/2016 19:04'!
disconnect

	self postNewMessage: (ChatMessage from: login text: '-- exited from the chat room --').
	exit := true

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 05:51'!
getLastMessageIndexAtStart

	| responseString |	
			
	[ responseString := self basicSecuredGet: self commandLastMessageIndex ] value
		ifNil: [ ^ lastMessageIndex ].
	^ self objectFromSTON: responseString
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 05:51'!
getNewMessages
	"Gets the new messages that have been posted since the last request."
	"Prints the new messages on the console."

	| responseString receivedChatMessages |
	
	^ [ responseString := self basicSecuredGet: self commandMessagesFromLastIndex ] value
		ifNil: [ 0 ]
		ifNotNil: [
			receivedChatMessages := self objectFromSTON: responseString.
			console print: receivedChatMessages.
			messages addAll: receivedChatMessages.
			receivedChatMessages size ]
	
! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 11:22'!
host: aHost port: aPort login: aLogin

	url := 'http://' , aHost , ':' , aPort asString.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/2/2016 15:32'!
initialize

	super initialize.
	exit := false.
	networkError := nil.
	lastMessageIndex := 0.
	messages := OrderedCollection new! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
login
	^ login! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 12:41'!
objectFromSTON: aByteString
	^ (STONReader on: aByteString readStream) next
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 13:05'!
postNewMessage: aChatMessage

	^ self basicSecuredPost: self commandAddMessage
		message: aChatMessage! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 19:06'!
printConnectionErrorMessage: aNetworkErrorString

	| errorMessage |
	
	aNetworkErrorString = networkError ifFalse: [ 
		networkError := aNetworkErrorString.
		errorMessage := ChatMessage 
			from: '-----' 
			text: 'Network Error :', networkError.
		console print: errorMessage asOrderedCollection ]


	! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/2/2016 11:39'!
refreshMessages
	"Create and schedule a light process to get new posted messages every 2 seconds."
	"At client disconnection the process ends."
	
	[
		[ exit ] whileFalse: [
			(Delay forSeconds: 2) wait.
			lastMessageIndex := lastMessageIndex + self getNewMessages ] 
	] fork! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/2/2016 11:37'!
send: aString
	"Post a message to the server and get back from it the list of all new posted messages."
	
	| newChatMessage |
	
	newChatMessage := ChatMessage from: login text: aString.
	self postNewMessage: newChatMessage.
	lastMessageIndex := lastMessageIndex + self getNewMessages
! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/4/2016 19:04'!
start

	console := ChatConsole attach: self.
	lastMessageIndex := self getLastMessageIndexAtStart.
	self postNewMessage: (ChatMessage from: login text: '-- joined the chat room --').
	self refreshMessages! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
url
	^ url! !

Object subclass: #ChatMessage
	instanceVariableNames: 'sender text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Message'!

!ChatMessage commentStamp: 'FrancoisJarriges 6/2/2016 14:57' prior: 0!
I'm a simple chat message. I identify my sender and my text.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	sender:		<Object>
	text:		<Object>


    Implementation Points!

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 08:59'!
from: aSender text: aText

	^ self new sender: aSender; text: aText; yourself
! !

!ChatMessage class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:54'!
separator
	^ '>'! !

!ChatMessage methodsFor: 'printing' stamp: 'FrancoisJarriges 6/1/2016 15:01'!
printOn: aStream

	aStream 
		<< self sender; 
		<< self class separator;
		<< String space;
		<< self text; 
		<< String crlf! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender
	^ sender! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
sender: aByteString
	sender := aByteString! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text
	^ text! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
text: aByteString
	text := aByteString! !

Object subclass: #ChatMessagesQueue
	instanceVariableNames: 'messages announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessagesQueue commentStamp: 'FrancoisJarriges 6/3/2016 19:46' prior: 0!
I represent a list of all the messages sent by clients in this chat application.

    Instance Variables
	messages:		<OrderedCollection>
	announcer: 		<Announcer>


    Implementation Points!

!ChatMessagesQueue methodsFor: 'adding' stamp: 'FrancoisJarriges 6/4/2016 03:59'!
add: aChatMessage
	messages add: aChatMessage 
! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:46'!
announcer
	
	^ announcer ifNil: [ announcer := Announcer new ]! !

!ChatMessagesQueue methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 04:33'!
initialize

	super initialize.
	messages := OrderedCollection new.
	self announcer 
		when: ChatMessageAnnouncement 
		do: [ :thisAnnouncement |
			self add: thisAnnouncement message ]
 ! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 03:21'!
listFrom: anIndex

	^ (anIndex between: 0 and: messages size - 1)
		ifTrue: [ messages copyFrom: anIndex + 1 to: messages size ]
		ifFalse: [ #() ]! !

!ChatMessagesQueue methodsFor: 'resetting' stamp: 'FrancoisJarriges 6/4/2016 04:13'!
reset
	self announcer unsubscribe: self.
	messages removeAll! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
size
	^ messages size! !

Object subclass: #ChatServer
	instanceVariableNames: 'teapotServer messagesQueue log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatServer commentStamp: 'FrancoisJarriges 6/4/2016 22:14' prior: 0!
I'm a chat server. I'm listening to HTTPRequests and managing a list of messages. 

My clients often request the messages I hold. Indeed, it may happen that client sends a lot of message and another one should request the difference between the messages it knows and the ones published on the server to be able to refresh correctly. 

I register some routes 
	/messages/ aNumber  get all the messages from aNumber to the end.
	/messages/count to get the current number of messages
	/messages/ add to add a message

ChatServer stopAll.
ChatServer startOn: 8080.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	teapotServer:		<Teapot>
	messagesQueue:		<ChatMessagesQueue>
	log			<ChatUsersLogged>


    Implementation Points
	As  my underlying ZnServer could spawn child threads to cope with concurrent POST requests:
	- in the context of a POST request treatment  a ChatMessageAnnoucement is made that communicates the message content to the ChatMessagesQueue announcer 
	- the actual writing operation of this new message to the ChatMessagesQueue is then realized in the context of the main server thread by the ChatMessagesQueue itself (that has subscribed to the ChatMessageAnnouncement at start)!

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/3/2016 15:01'!
startOn: aPortNumber

	^ self new
		initializePort: aPortNumber;
		registerRoutes;
		registerErrorHandlers;
		start;
		yourself! !

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:44'!
stopAll

	^ self allInstancesDo: #stop! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/4/2016 23:44'!
addMessage: aTeaRequest

	| newChatMessage thisChatUserId |
	
	newChatMessage := (STONReader on: aTeaRequest entity string readStream) next.
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	log users at: thisChatUserId asSymbol 
		ifAbsentPut: [ ChatUser withLogin: newChatMessage sender].

	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: newChatMessage)
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:13'!
initialize

	super initialize.
	messagesQueue := ChatMessagesQueue new.
	log := ChatUsersLogged  new
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 15:25'!
initializePort: anInteger

	teapotServer := Teapot configure: {
		#defaultOutput -> #text.
		#port -> anInteger.
		#debugMode -> true }
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/4/2016 23:45'!
messageCount

	^ messagesQueue size! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/4/2016 23:47'!
messagesFrom: aTeaRequest

	| thisChatUserId |
	
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	(log users at: thisChatUserId asSymbol) lifeSign: true.
	
	^ messagesQueue listFrom: (aTeaRequest at: #id)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 10:12'!
registerErrorHandlers
	
	teapotServer 
		exception: KeyNotFound -> (TeaResponse notFound body: 'No such message')! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 23:45'!
registerRoutes
	
	teapotServer 
		GET: '/messages/count' -> (Send message: #messageCount to: self);output: #ston;
		GET: '/messages/<id:IsInteger>' -> (Send message: #messagesFrom: to: self); output: #ston;
		POST: '/messages/add' -> (Send message: #addMessage: to: self)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 23:49'!
start
		
	teapotServer start.
	log updateConnections 
	

! !

!ChatServer methodsFor: 'public' stamp: 'FrancoisJarriges 6/4/2016 23:50'!
stop
	
	teapotServer stop.
	log stopUpdateConnections.
	messagesQueue reset! !

Object subclass: #ChatUser
	instanceVariableNames: 'login connected lifeSign'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-User'!

!ChatUser commentStamp: 'FrancoisJarriges 6/4/2016 17:31' prior: 0!
I represent a user in of this tiny chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables

	login:		<ByteString>


    Implementation Points!

!ChatUser class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/4/2016 22:06'!
withLogin: loginString
	
	^ self new 
		login: loginString;
		connected: true;
		lifeSign: true;
		yourself! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
connected
	^ connected ifNil: [ connected := false ]! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
connected: aBoolean
	connected := aBoolean! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
lifeSign
	^ lifeSign ifNil: [ lifeSign := false ]! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:04'!
lifeSign: aBoolean
	lifeSign := aBoolean! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 07:29'!
login
	^ login ifNil: [ login := '' ]! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:59'!
login: aByteString
	login := aByteString! !

!ChatUser methodsFor: 'printing' stamp: 'FrancoisJarriges 6/4/2016 17:31'!
printOn: aStream

	aStream 
		<< self login; 
		<< String crlf! !

Object subclass: #ChatUsersLogged
	instanceVariableNames: 'users checkConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatUsersLogged commentStamp: 'FrancoisJarriges 6/4/2016 22:09' prior: 0!
I represent all the users connections to the chat since the server started.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	users:				<Dictionary>
	checkConnections: 	<Boolean>


    Implementation Points
	!

!ChatUsersLogged methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:21'!
initialize

	super initialize.
	users := Dictionary new.
	checkConnections := true! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 23:50'!
stopUpdateConnections
	checkConnections := false! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 23:49'!
updateConnections
	"Create and schedule a light process to check users connections live signs every 30 seconds."
	"When the server stops the process ends."
	
	[
		[ checkConnections ] whileTrue: [
			(Delay forSeconds: 30) wait.
			 users do: [ :eachUser | 
				eachUser connected: eachUser lifeSign.
				eachUser lifeSign: false ] ]
	] fork! !

!ChatUsersLogged methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 10:17'!
users
	^ users! !

Announcement subclass: #ChatMessageAnnouncement
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessageAnnouncement commentStamp: 'FrancoisJarriges 6/3/2016 19:41' prior: 0!
I represent a new message  event  raised from a server thread at each reception of a new message post.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	message:		<ChatMessage>


    Implementation Points!

!ChatMessageAnnouncement class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/3/2016 19:54'!
for: aChatMessage
	
	 ^ self new message: aChatMessage! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 20:34'!
message

	^ message! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:55'!
message: aChatMessage

	message := aChatMessage! !

ComposableModel subclass: #ChatConsole
	instanceVariableNames: 'chat list input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsole commentStamp: 'FrancoisJarriges 6/2/2016 15:00' prior: 0!
I represent the console window to be used for the chat application to write and read messages.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chat:		<Object>
	input:		<Object>
	list:		<Object>


    Implementation Points!

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:40'!
attach: aChatClient

	| aChatConsole |
	
	aChatConsole := self new chat: aChatClient.
	aChatConsole openWithSpec whenClosedDo: [ aChatClient disconnect ].
	^ aChatConsole! !

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:53'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
			newColumn: [ :c |
				c add: #list; add: #input height: 30 ]; yourself
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:32'!
chat: aChatClient
	chat := aChatClient! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 10:56'!
initializeWidgets

	list := ChatConsoleListModel new.
	input := TextInputFieldModel new
		ghostText: 'Type your message here...';
		enabled: true;
		acceptBlock: [ :inputString |
			chat send: inputString.
			input text: '' ].
	self focusOrder add: input
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
input
	^ input! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
list
	^ list! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:57'!
print: aCollectionOfChatMessages

	list addItems: (aCollectionOfChatMessages collect: [ :eachChatMessage | 
		eachChatMessage ])
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 17:08'!
title
	^ chat login, ' on ', chat url! !

ListModel subclass: #ChatConsoleListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsoleListModel commentStamp: 'FrancoisJarriges 6/2/2016 15:06' prior: 0!
I specialize the standard UI ListModel class for a chat console window.


    Implementation Points!

!ChatConsoleListModel methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:55'!
addItems: aList
	"Add a new list of items to the existing list."

	| oldContents |
	oldContents := listHolder value.
	listHolder value: {oldContents . (aList sorted: sortingBlockHolder value)} concatenation.
	listAnnouncer announce: (ValueChanged oldValue: oldContents newValue: listHolder value).
	! !
